@@ -5,10 +5,9 @@ from trac.ticket.api import ITicketActionController
 from trac.ticket.default_workflow import ConfigurableTicketWorkflow
 from trac.perm import IPermissionRequestor
 from trac.config import Option, ListOption
-from trac.util.compat import set
 
-revision = "$Rev: 6326 $"
-url = "$URL: http://svn.edgewall.org/repos/trac/tags/trac-0.11.7/sample-plugins/workflow/CodeReview.py $"
+revision = "$Rev: 7457 $"
+url = "$URL: https://svn.edgewall.org/repos/trac/tags/trac-0.12/sample-plugins/workflow/CodeReview.py $"
 
 class CodeReviewActionController(Component):
     """Support for simple code reviews.
