@@ -5,7 +5,7 @@ from trac.ticket.api import ITicketActionController, TicketSystem
 from trac.perm import IPermissionRequestor
 
 revision = "$Rev: 6326 $"
-url = "$URL: http://svn.edgewall.org/repos/trac/tags/trac-0.11.7/sample-plugins/workflow/StatusFixer.py $"
+url = "$URL: https://svn.edgewall.org/repos/trac/tags/trac-0.12/sample-plugins/workflow/StatusFixer.py $"
 
 class StatusFixerActionController(Component):
     """Provides the admin with a way to correct a ticket's status.
