@@ -15,10 +15,10 @@
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
 from datetime import datetime
+import new
 import os.path
 import stat
 import shutil
-import sys
 import tempfile
 import unittest
 
@@ -31,16 +31,22 @@ except:
     has_svn = False
 
 from trac.log import logger_factory
-from trac.test import TestSetup
+from trac.test import EnvironmentStub, TestSetup
 from trac.core import TracError
+from trac.resource import Resource, resource_exists
+from trac.util.concurrency import get_thread_id
 from trac.util.datefmt import utc
-from trac.versioncontrol import Changeset, Node, NoSuchChangeset
-from trac.versioncontrol.svn_fs import SubversionRepository
+from trac.versioncontrol import DbRepositoryProvider, Changeset, Node, \
+                                NoSuchChangeset
+from trac.versioncontrol.svn_fs import SvnCachedRepository, \
+                                       SubversionRepository
 from trac.versioncontrol import svn_fs
 
 REPOS_PATH = os.path.join(tempfile.gettempdir(), 'trac-svnrepos')
+REPOS_NAME = 'repo'
 
-HEAD = 21
+HEAD = 22
+TETE = 21
 
 
 class SubversionRepositoryTestSetup(TestSetup):
@@ -54,6 +60,8 @@ class SubversionRepositoryTestSetup(TestSetup):
         pool = core.svn_pool_create(None)
         dumpstream = None
         try:
+            if os.path.exists(REPOS_PATH):
+                print 'trouble ahead with db/rep-cache.db... see #8278'
             r = repos.svn_repos_create(REPOS_PATH, '', '', None, None, pool)
             if hasattr(repos, 'svn_repos_load_fs2'):
                 repos.svn_repos_load_fs2(r, dumpfile, StringIO(),
@@ -81,15 +89,21 @@ class SubversionRepositoryTestSetup(TestSetup):
         shutil.rmtree(REPOS_PATH)
 
 
-class SubversionRepositoryTestCase(unittest.TestCase):
+# -- Re-usable test mixins
 
-    def setUp(self):
-        self.repos = SubversionRepository(REPOS_PATH, None,
-                                          logger_factory('test'))
-
-    def tearDown(self):
-        self.repos = None
+class NormalTests(object):
 
+    def test_resource_exists(self):
+        repos = Resource('repository', REPOS_NAME)
+        self.assertEqual(True, resource_exists(self.env, repos))
+        self.assertEqual(False, resource_exists(self.env, repos(id='xxx')))
+        node = repos.child('source', u'tête')
+        self.assertEqual(True, resource_exists(self.env, node))
+        self.assertEqual(False, resource_exists(self.env, node(id='xxx')))
+        cset = repos.child('changeset', HEAD)
+        self.assertEqual(True, resource_exists(self.env, cset))
+        self.assertEqual(False, resource_exists(self.env, cset(id=123456)))
+                         
     def test_repos_normalize_path(self):
         self.assertEqual('/', self.repos.normalize_path('/'))
         self.assertEqual('/', self.repos.normalize_path(''))
@@ -130,7 +144,8 @@ class SubversionRepositoryTestCase(unittest.TestCase):
         # ...
         self.assertEqual(None, self.repos.next_rev(HEAD, u'tête'))
         # test accentuated characters
-        self.assertEqual(None, self.repos.previous_rev(17, u'tête/R\xe9sum\xe9.txt'))
+        self.assertEqual(None,
+                         self.repos.previous_rev(17, u'tête/R\xe9sum\xe9.txt'))
         self.assertEqual(17, self.repos.next_rev(16, u'tête/R\xe9sum\xe9.txt'))
 
     def test_has_node(self):
@@ -143,15 +158,16 @@ class SubversionRepositoryTestCase(unittest.TestCase):
         self.assertEqual(u'tête', node.name)
         self.assertEqual(u'/tête', node.path)
         self.assertEqual(Node.DIRECTORY, node.kind)
-        self.assertEqual(HEAD, node.rev)
-        self.assertEqual(datetime(2007,4,30,17,45,26,0,utc),
+        self.assertEqual(TETE, node.rev)
+        self.assertEqual(datetime(2007, 4, 30, 17, 45, 26, 234375, utc),
                          node.last_modified)
         node = self.repos.get_node(u'/tête/README.txt')
         self.assertEqual('README.txt', node.name)
         self.assertEqual(u'/tête/README.txt', node.path)
         self.assertEqual(Node.FILE, node.kind)
         self.assertEqual(3, node.rev)
-        self.assertEqual(datetime(2005,4,1,13,24,58,0,utc), node.last_modified)
+        self.assertEqual(datetime(2005, 4, 1, 13, 24, 58, 234643, utc),
+                         node.last_modified)
 
     def test_get_node_specific_rev(self):
         node = self.repos.get_node(u'/tête', 1)
@@ -159,13 +175,15 @@ class SubversionRepositoryTestCase(unittest.TestCase):
         self.assertEqual(u'/tête', node.path)
         self.assertEqual(Node.DIRECTORY, node.kind)
         self.assertEqual(1, node.rev)
-        self.assertEqual(datetime(2005,4,1,10,0,52,0,utc), node.last_modified)
+        self.assertEqual(datetime(2005, 4, 1, 10, 0, 52, 353248, utc),
+                         node.last_modified)
         node = self.repos.get_node(u'/tête/README.txt', 2)
         self.assertEqual('README.txt', node.name)
         self.assertEqual(u'/tête/README.txt', node.path)
         self.assertEqual(Node.FILE, node.kind)
         self.assertEqual(2, node.rev)
-        self.assertEqual(datetime(2005,4,1,13,12,18,0,utc), node.last_modified)
+        self.assertEqual(datetime(2005, 4, 1, 13, 12, 18, 216267, utc),
+                         node.last_modified)
 
     def test_get_dir_entries(self):
         node = self.repos.get_node(u'/tête')
@@ -231,6 +249,13 @@ class SubversionRepositoryTestCase(unittest.TestCase):
         self.assertEqual((u'tête/README.txt', 2, 'add'), history.next())
         self.assertRaises(StopIteration, history.next)
 
+    def test_get_node_history_limit(self):
+        node = self.repos.get_node(u'/tête/README3.txt')
+        history = node.get_history(2)
+        self.assertEqual((u'tête/README3.txt', 14, 'copy'), history.next())
+        self.assertEqual((u'tête/README2.txt', 6, 'copy'), history.next())
+        self.assertRaises(StopIteration, history.next)
+
     def test_get_node_history_follow_copy(self):
         node = self.repos.get_node('/tags/v1/README.txt')
         history = node.get_history()
@@ -239,6 +264,46 @@ class SubversionRepositoryTestCase(unittest.TestCase):
         self.assertEqual((u'tête/README.txt', 2, 'add'), history.next())
         self.assertRaises(StopIteration, history.next)
 
+    def test_get_copy_ancestry(self):
+        node = self.repos.get_node('/tags/v1/README.txt')
+        ancestry = node.get_copy_ancestry()
+        self.assertEqual([(u'tête/README.txt', 6)], ancestry)
+        for path, rev in ancestry:
+            self.repos.get_node(path, rev) # shouldn't raise NoSuchNode
+
+        node = self.repos.get_node(u'/tête/README3.txt')
+        ancestry = node.get_copy_ancestry()
+        self.assertEqual([(u'tête/README2.txt', 13), 
+                          (u'tête/README.txt', 3)], ancestry)
+        for path, rev in ancestry:
+            self.repos.get_node(path, rev) # shouldn't raise NoSuchNode
+
+        node = self.repos.get_node('/branches/v1x')
+        ancestry = node.get_copy_ancestry()
+        self.assertEqual([(u'tags/v1.1', 11),
+                          (u'branches/v1x', 9), 
+                          (u'tags/v1', 7), 
+                          (u'tête', 6)], ancestry)
+        for path, rev in ancestry:
+            self.repos.get_node(path, rev) # shouldn't raise NoSuchNode
+
+    def test_get_copy_ancestry_for_move(self):
+        node = self.repos.get_node(u'/tête/dir1/dir2', 5)
+        ancestry = node.get_copy_ancestry()
+        self.assertEqual([(u'tête/dir2', 4)], ancestry)
+        for path, rev in ancestry:
+            self.repos.get_node(path, rev) # shouldn't raise NoSuchNode
+
+    def test_get_branch_origin(self):
+        node = self.repos.get_node('/tags/v1/README.txt')
+        self.assertEqual(7, node.get_branch_origin())
+        node = self.repos.get_node(u'/tête/README3.txt')
+        self.assertEqual(14, node.get_branch_origin())
+        node = self.repos.get_node('/branches/v1x')
+        self.assertEqual(12, node.get_branch_origin())
+        node = self.repos.get_node(u'/tête/dir1/dir2', 5)
+        self.assertEqual(5, node.get_branch_origin())
+
     # Revision Log / path history 
 
     def test_get_path_history(self):
@@ -276,7 +341,8 @@ class SubversionRepositoryTestCase(unittest.TestCase):
         self.assertEqual(expected[2], (got[2], got[3]))
         
     def test_diff_file_different_revs(self):
-        diffs = self.repos.get_changes(u'tête/README.txt', 2, u'tête/README.txt', 3)
+        diffs = self.repos.get_changes(u'tête/README.txt', 2,
+                                       u'tête/README.txt', 3)
         self._cmp_diff(((u'tête/README.txt', 2),
                         (u'tête/README.txt', 3),
                         (Node.FILE, Changeset.EDIT)), diffs.next())
@@ -335,7 +401,8 @@ class SubversionRepositoryTestCase(unittest.TestCase):
         self.assertEqual(0, chgset.rev)
         self.assertEqual('', chgset.message)
         self.assertEqual('', chgset.author)
-        self.assertEqual(datetime(2005,4,1,9,57,41,0,utc), chgset.date)
+        self.assertEqual(datetime(2005, 4, 1, 9, 57, 41, 312767, utc),
+                         chgset.date)
         self.assertRaises(StopIteration, chgset.get_changes().next)
 
     def test_changeset_added_dirs(self):
@@ -343,7 +410,8 @@ class SubversionRepositoryTestCase(unittest.TestCase):
         self.assertEqual(1, chgset.rev)
         self.assertEqual('Initial directory layout.', chgset.message)
         self.assertEqual('john', chgset.author)
-        self.assertEqual(datetime(2005,4,1,10,0,52,0,utc), chgset.date)
+        self.assertEqual(datetime(2005, 4, 1, 10, 0, 52, 353248, utc),
+                         chgset.date)
 
         changes = chgset.get_changes()
         self.assertEqual(('branches', Node.DIRECTORY, Changeset.ADD, None, -1),
@@ -359,7 +427,8 @@ class SubversionRepositoryTestCase(unittest.TestCase):
         self.assertEqual(3, chgset.rev)
         self.assertEqual('Fixed README.\n', chgset.message)
         self.assertEqual('kate', chgset.author)
-        self.assertEqual(datetime(2005,4,1,13,24,58,0,utc), chgset.date)
+        self.assertEqual(datetime(2005, 4, 1, 13, 24, 58, 234643, utc),
+                         chgset.date)
 
         changes = chgset.get_changes()
         self.assertEqual((u'tête/README.txt', Node.FILE, Changeset.EDIT,
@@ -371,7 +440,8 @@ class SubversionRepositoryTestCase(unittest.TestCase):
         self.assertEqual(5, chgset.rev)
         self.assertEqual('Moved directories.', chgset.message)
         self.assertEqual('kate', chgset.author)
-        self.assertEqual(datetime(2005,4,1,16,25,39,0,utc), chgset.date)
+        self.assertEqual(datetime(2005, 4, 1, 16, 25, 39, 658099, utc),
+                         chgset.date)
 
         changes = chgset.get_changes()
         self.assertEqual((u'tête/dir1/dir2', Node.DIRECTORY, Changeset.MOVE,
@@ -385,7 +455,8 @@ class SubversionRepositoryTestCase(unittest.TestCase):
         self.assertEqual(6, chgset.rev)
         self.assertEqual('More things to read', chgset.message)
         self.assertEqual('john', chgset.author)
-        self.assertEqual(datetime(2005,4,1,18,56,46,0,utc), chgset.date)
+        self.assertEqual(datetime(2005, 4, 1, 18, 56, 46, 985846, utc),
+                         chgset.date)
 
         changes = chgset.get_changes()
         self.assertEqual((u'tête/README2.txt', Node.FILE, Changeset.COPY,
@@ -438,16 +509,35 @@ class SubversionRepositoryTestCase(unittest.TestCase):
         chgset = self.repos.get_changeset(19)
         self.assertEqual(19, chgset.rev)
         changes = chgset.get_changes()
-        self.assertEqual((u'tête/Xprimary_proc/Xprimary_pkg.vhd', Node.FILE,
-                          Changeset.DELETE,
-                          u'tête/Xprimary_proc/Xprimary_pkg.vhd', 18),
-                         changes.next())
         self.assertEqual((u'tête/mpp_proc', Node.DIRECTORY,
                           Changeset.MOVE, u'tête/Xprimary_proc', 18),
                          changes.next())
+        self.assertEqual((u'tête/mpp_proc/Xprimary_pkg.vhd',
+                          Node.FILE, Changeset.DELETE,
+                          u'tête/Xprimary_proc/Xprimary_pkg.vhd', 18),
+                         changes.next())
         self.assertEqual((u'tête/mpp_proc/Xprimary_proc', Node.DIRECTORY,
                           Changeset.COPY, u'tête/Xprimary_proc', 18),
                          changes.next())
+        self.assertEqual((u'tête/mpp_proc/Xprimary_proc/Xprimary_pkg.vhd',
+                          Node.FILE, Changeset.DELETE,
+                          u'tête/Xprimary_proc/Xprimary_pkg.vhd', 18),
+                         changes.next())
+        self.assertRaises(StopIteration, changes.next)
+
+    def test_copy_with_deletions_below_copy(self):
+        """Regression test for #4900."""
+        chgset = self.repos.get_changeset(22)
+        self.assertEqual(22, chgset.rev)
+        changes = chgset.get_changes()
+        self.assertEqual((u'branches/v3', 'dir', 'copy',
+                          u'tête', 21), changes.next())
+        self.assertEqual((u'branches/v3/dir1', 'dir', 'delete',
+                          u'tête/dir1', 21), changes.next())
+        self.assertEqual((u'branches/v3/mpp_proc', 'dir', 'delete',
+                          u'tête/mpp_proc', 21), changes.next())
+        self.assertEqual((u'branches/v3/v2', 'dir', 'delete',
+                          u'tête/v2', 21), changes.next())
         self.assertRaises(StopIteration, changes.next)
 
     def test_changeset_utf_8(self):
@@ -456,14 +546,9 @@ class SubversionRepositoryTestCase(unittest.TestCase):
         self.assertEqual(u'Chez moi ça marche\n', chgset.message)
         self.assertEqual(u'Jonas Borgström', chgset.author)
 
-class ScopedSubversionRepositoryTestCase(unittest.TestCase):
 
-    def setUp(self):
-        self.repos = SubversionRepository(REPOS_PATH + u'/tête', None,
-                                          logger_factory('test'))
 
-    def tearDown(self):
-        self.repos = None
+class ScopedTests(object):
 
     def test_repos_normalize_path(self):
         self.assertEqual('/', self.repos.normalize_path('/'))
@@ -475,10 +560,10 @@ class ScopedSubversionRepositoryTestCase(unittest.TestCase):
         self.assertEqual('dir1', self.repos.normalize_path('/dir1/'))
 
     def test_repos_normalize_rev(self):
-        self.assertEqual(HEAD, self.repos.normalize_rev('latest'))
-        self.assertEqual(HEAD, self.repos.normalize_rev('head'))
-        self.assertEqual(HEAD, self.repos.normalize_rev(''))
-        self.assertEqual(HEAD, self.repos.normalize_rev(None))
+        self.assertEqual(TETE, self.repos.normalize_rev('latest'))
+        self.assertEqual(TETE, self.repos.normalize_rev('head'))
+        self.assertEqual(TETE, self.repos.normalize_rev(''))
+        self.assertEqual(TETE, self.repos.normalize_rev(None))
         self.assertEqual(5, self.repos.normalize_rev('5'))
         self.assertEqual(5, self.repos.normalize_rev(5))
 
@@ -486,11 +571,11 @@ class ScopedSubversionRepositoryTestCase(unittest.TestCase):
         self.assertEqual(1, self.repos.oldest_rev)
         self.assertEqual(None, self.repos.previous_rev(0))
         self.assertEqual(1, self.repos.previous_rev(2))
-        self.assertEqual(HEAD, self.repos.youngest_rev)
+        self.assertEqual(TETE, self.repos.youngest_rev)
         self.assertEqual(2, self.repos.next_rev(1))
         self.assertEqual(3, self.repos.next_rev(2))
         # ...
-        self.assertEqual(None, self.repos.next_rev(HEAD))
+        self.assertEqual(None, self.repos.next_rev(TETE))
 
     def test_has_node(self):
         self.assertEqual(False, self.repos.has_node('/dir1', 3))
@@ -502,13 +587,15 @@ class ScopedSubversionRepositoryTestCase(unittest.TestCase):
         self.assertEqual('/dir1', node.path)
         self.assertEqual(Node.DIRECTORY, node.kind)
         self.assertEqual(5, node.rev)
-        self.assertEqual(datetime(2005,4,1,16,25,39,0,utc), node.last_modified)
+        self.assertEqual(datetime(2005, 4, 1, 16, 25, 39, 658099, utc),
+                         node.last_modified)
         node = self.repos.get_node('/README.txt')
         self.assertEqual('README.txt', node.name)
         self.assertEqual('/README.txt', node.path)
         self.assertEqual(Node.FILE, node.kind)
         self.assertEqual(3, node.rev)
-        self.assertEqual(datetime(2005,4,1,13,24,58,0,utc), node.last_modified)
+        self.assertEqual(datetime(2005, 4, 1, 13, 24, 58, 234643, utc),
+                         node.last_modified)
 
     def test_get_node_specific_rev(self):
         node = self.repos.get_node('/dir1', 4)
@@ -516,13 +603,15 @@ class ScopedSubversionRepositoryTestCase(unittest.TestCase):
         self.assertEqual('/dir1', node.path)
         self.assertEqual(Node.DIRECTORY, node.kind)
         self.assertEqual(4, node.rev)
-        self.assertEqual(datetime(2005,4,1,15,42,35,0,utc), node.last_modified)
+        self.assertEqual(datetime(2005, 4, 1, 15, 42, 35, 450595, utc),
+                         node.last_modified)
         node = self.repos.get_node('/README.txt', 2)
         self.assertEqual('README.txt', node.name)
         self.assertEqual('/README.txt', node.path)
         self.assertEqual(Node.FILE, node.kind)
         self.assertEqual(2, node.rev)
-        self.assertEqual(datetime(2005,4,1,13,12,18,0,utc), node.last_modified)
+        self.assertEqual(datetime(2005, 4, 1, 13, 12, 18, 216267, utc),
+                         node.last_modified)
 
     def test_get_dir_entries(self):
         node = self.repos.get_node('/')
@@ -581,6 +670,27 @@ class ScopedSubversionRepositoryTestCase(unittest.TestCase):
         self.assertEqual(('dir3', 4, 'add'), history.next())
         self.assertRaises(StopIteration, history.next)
 
+    def test_get_copy_ancestry(self):
+        node = self.repos.get_node(u'/README3.txt')
+        ancestry = node.get_copy_ancestry()
+        self.assertEqual([(u'README2.txt', 13), 
+                          (u'README.txt', 3)], ancestry)
+        for path, rev in ancestry:
+            self.repos.get_node(path, rev) # shouldn't raise NoSuchNode
+
+    def test_get_copy_ancestry_for_move(self):
+        node = self.repos.get_node(u'/dir1/dir2', 5)
+        ancestry = node.get_copy_ancestry()
+        self.assertEqual([(u'dir2', 4)], ancestry)
+        for path, rev in ancestry:
+            self.repos.get_node(path, rev) # shouldn't raise NoSuchNode
+
+    def test_get_branch_origin(self):
+        node = self.repos.get_node(u'/README3.txt')
+        self.assertEqual(14, node.get_branch_origin())
+        node = self.repos.get_node(u'/dir1/dir2', 5)
+        self.assertEqual(5, node.get_branch_origin())
+
     # Revision Log / path history 
 
     def test_get_path_history(self):
@@ -606,7 +716,8 @@ class ScopedSubversionRepositoryTestCase(unittest.TestCase):
         self.assertEqual(0, chgset.rev)
         self.assertEqual('', chgset.message)
         self.assertEqual('', chgset.author)
-        self.assertEqual(datetime(2005,4,1,9,57,41,0,utc), chgset.date)
+        self.assertEqual(datetime(2005, 4, 1, 9, 57, 41, 312767, utc),
+                         chgset.date)
         self.assertRaises(StopIteration, chgset.get_changes().next)
 
     def test_changeset_added_dirs(self):
@@ -614,7 +725,8 @@ class ScopedSubversionRepositoryTestCase(unittest.TestCase):
         self.assertEqual(4, chgset.rev)
         self.assertEqual('More directories.', chgset.message)
         self.assertEqual('john', chgset.author)
-        self.assertEqual(datetime(2005,4,1,15,42,35,0,utc), chgset.date)
+        self.assertEqual(datetime(2005, 4, 1, 15, 42, 35, 450595, utc),
+                         chgset.date)
 
         changes = chgset.get_changes()
         self.assertEqual(('dir1', Node.DIRECTORY, 'add', None, -1),
@@ -630,7 +742,8 @@ class ScopedSubversionRepositoryTestCase(unittest.TestCase):
         self.assertEqual(3, chgset.rev)
         self.assertEqual('Fixed README.\n', chgset.message)
         self.assertEqual('kate', chgset.author)
-        self.assertEqual(datetime(2005,4,1,13,24,58,0,utc), chgset.date)
+        self.assertEqual(datetime(2005, 4, 1, 13, 24, 58, 234643, utc),
+                         chgset.date)
 
         changes = chgset.get_changes()
         self.assertEqual(('README.txt', Node.FILE, Changeset.EDIT,
@@ -642,7 +755,8 @@ class ScopedSubversionRepositoryTestCase(unittest.TestCase):
         self.assertEqual(5, chgset.rev)
         self.assertEqual('Moved directories.', chgset.message)
         self.assertEqual('kate', chgset.author)
-        self.assertEqual(datetime(2005,4,1,16,25,39,0,utc), chgset.date)
+        self.assertEqual(datetime(2005, 4, 1, 16, 25, 39, 658099, utc),
+                         chgset.date)
 
         changes = chgset.get_changes()
         self.assertEqual(('dir1/dir2', Node.DIRECTORY, Changeset.MOVE,
@@ -656,7 +770,8 @@ class ScopedSubversionRepositoryTestCase(unittest.TestCase):
         self.assertEqual(6, chgset.rev)
         self.assertEqual('More things to read', chgset.message)
         self.assertEqual('john', chgset.author)
-        self.assertEqual(datetime(2005,4,1,18,56,46,0,utc), chgset.date)
+        self.assertEqual(datetime(2005, 4, 1, 18, 56, 46, 985846, utc),
+                         chgset.date)
 
         changes = chgset.get_changes()
         self.assertEqual(('README2.txt', Node.FILE, Changeset.COPY,
@@ -679,19 +794,16 @@ class ScopedSubversionRepositoryTestCase(unittest.TestCase):
         self.assertEqual('copy from outside of the scope + delete',
                          chgset.message)
         changes = chgset.get_changes()
-        self.assertEqual(('v2', 'dir', Changeset.ADD, None, -1),
+        self.assertEqual(('v2', Node.DIRECTORY, Changeset.ADD, None, -1),
                          changes.next())
+        self.assertEqual(('v2/README2.txt', Node.FILE, Changeset.DELETE,
+                          None, -1), changes.next())
+        self.assertEqual(('v2/dir1', Node.DIRECTORY, Changeset.DELETE,
+                          None, -1), changes.next())
         self.assertRaises(StopIteration, changes.next)
 
 
-class RecentPathScopedRepositoryTestCase(unittest.TestCase):
-
-    def setUp(self):
-        self.repos = SubversionRepository(REPOS_PATH + u'/tête/dir1', None,
-                                          logger_factory('test'))
-
-    def tearDown(self):
-        self.repos = None
+class RecentPathScopedTests(object):
 
     def test_rev_navigation(self):
         self.assertEqual(False, self.repos.has_node('/', 1))
@@ -704,14 +816,7 @@ class RecentPathScopedRepositoryTestCase(unittest.TestCase):
         self.assertEqual(None, self.repos.previous_rev(4))
 
 
-class NonSelfContainedScopedTestCase(unittest.TestCase):
-
-    def setUp(self):
-        self.repos = SubversionRepository(REPOS_PATH + '/tags/v1', None,
-                                          logger_factory('test'))
-
-    def tearDown(self):
-        self.repos = None
+class NonSelfContainedScopedTests(object):
 
     def test_mixed_changeset(self):
         chgset = self.repos.get_changeset(7)
@@ -722,14 +827,7 @@ class NonSelfContainedScopedTestCase(unittest.TestCase):
         self.assertRaises(TracError, lambda: self.repos.get_node(None, 6))
 
 
-class AnotherNonSelfContainedScopedTestCase(unittest.TestCase):
-
-    def setUp(self):
-        self.repos = SubversionRepository(REPOS_PATH + '/branches', None,
-                                          logger_factory('test'))
-
-    def tearDown(self):
-        self.repos = None
+class AnotherNonSelfContainedScopedTests(object):
 
     def test_mixed_changeset_with_edit(self):
         chgset = self.repos.get_changeset(9)
@@ -740,20 +838,72 @@ class AnotherNonSelfContainedScopedTestCase(unittest.TestCase):
                          changes.next())
 
 
+# -- Test cases for SubversionRepository
+
+class SubversionRepositoryTestCase(unittest.TestCase):
+    
+    def setUp(self):
+        self.env = EnvironmentStub()
+        repositories = self.env.config['repositories']
+        DbRepositoryProvider(self.env).add_repository(REPOS_NAME, self.path,
+                                                      'direct-svnfs')
+        self.repos = self.env.get_repository(REPOS_NAME)
+
+    def tearDown(self):
+        self.env.reset_db()
+        # needed to avoid issue with 'WindowsError: The process cannot access
+        # the file ... being used by another process: ...\rep-cache.db'
+        self.env.shutdown(get_thread_id())
+
+
+# -- Test cases for SvnCachedRepository
+
+class SvnCachedRepositoryTestCase(unittest.TestCase):
+    
+    def setUp(self):
+        self.env = EnvironmentStub()
+        DbRepositoryProvider(self.env).add_repository(REPOS_NAME, self.path,
+                                                      'svn')
+        self.repos = self.env.get_repository(REPOS_NAME)
+        self.repos.sync()
+
+    def tearDown(self):
+        self.env.reset_db()
+        self.repos.close()
+        self.repos = None
+
+
 def suite():
-    global has_svn
     suite = unittest.TestSuite()
     if has_svn:
-        suite.addTest(unittest.makeSuite(SubversionRepositoryTestCase,
-            'test', suiteClass=SubversionRepositoryTestSetup))
-        suite.addTest(unittest.makeSuite(ScopedSubversionRepositoryTestCase,
-            'test', suiteClass=SubversionRepositoryTestSetup))
-        suite.addTest(unittest.makeSuite(RecentPathScopedRepositoryTestCase,
-            'test', suiteClass=SubversionRepositoryTestSetup))
-        suite.addTest(unittest.makeSuite(NonSelfContainedScopedTestCase,
-            'test', suiteClass=SubversionRepositoryTestSetup))
-        suite.addTest(unittest.makeSuite(AnotherNonSelfContainedScopedTestCase,
-            'test', suiteClass=SubversionRepositoryTestSetup))
+        tests = [(NormalTests, ''),
+                 (ScopedTests, u'/tête'),
+                 (RecentPathScopedTests, u'/tête/dir1'),
+                 (NonSelfContainedScopedTests, '/tags/v1'),
+                 (AnotherNonSelfContainedScopedTests, '/branches'),
+                 ]
+        skipped = {
+            'SvnCachedRepositoryNormalTests': [
+                'test_changeset_repos_creation',
+                ],
+            'SvnCachedRepositoryScopedTests': [
+                'test_changeset_repos_creation',
+                'test_rev_navigation',
+                ],
+            }
+        for test, scope in tests:
+            tc = new.classobj('SubversionRepository' + test.__name__,
+                              (SubversionRepositoryTestCase, test),
+                              {'path': REPOS_PATH + scope})
+            suite.addTest(unittest.makeSuite(
+                tc, 'test', suiteClass=SubversionRepositoryTestSetup))
+            tc = new.classobj('SvnCachedRepository' + test.__name__,
+                              (SvnCachedRepositoryTestCase, test),
+                              {'path': REPOS_PATH + scope})
+            for skip in skipped.get(tc.__name__, []):
+                setattr(tc, skip, lambda self: None) # no skip, so we cheat...
+            suite.addTest(unittest.makeSuite(
+                tc, 'test', suiteClass=SubversionRepositoryTestSetup))
     else:
         print "SKIP: versioncontrol/tests/svn_fs.py (no svn bindings)"
     return suite
