@@ -3,14 +3,15 @@
 import unittest
 
 from trac.test import Mock
-from trac.wiki.tests import formatter
 from trac.versioncontrol import NoSuchChangeset
+from trac.versioncontrol.api import *
 from trac.versioncontrol.web_ui import *
+from trac.wiki.tests import formatter
 
 
 def _get_changeset(rev):
     if rev == '1':
-        return Mock(message="start")
+        return Mock(message="start", can_view=lambda perm: True)
     else:
         raise NoSuchChangeset(rev)
 
@@ -23,15 +24,17 @@ def _normalize_rev(rev):
         else:
             raise NoSuchChangeset(rev)
     
-def _get_repository(authname=None):
-    return Mock(get_changeset=_get_changeset, youngest_rev='200',
+def _get_repository(reponame):
+    return Mock(reponame=reponame, youngest_rev='200',
+                get_changeset=_get_changeset,
                 normalize_rev=_normalize_rev)
 
 def repository_setup(tc):
     setattr(tc.env, 'get_repository', _get_repository)
+    setattr(RepositoryManager(tc.env), 'get_repository', _get_repository)
 
 
-CHANGESET_TEST_CASES=u"""
+CHANGESET_TEST_CASES = u"""
 ============================== changeset: link resolver
 changeset:1
 changeset:12
@@ -126,14 +129,16 @@ T:r2081
 """ #"
 
 
-LOG_TEST_CASES=u"""
+LOG_TEST_CASES = u"""
 ============================== Log range TracLinks
 [1:2], r1:2, [12:23], r12:23
 [1:2/trunk]
+[2:1/trunk] reversed
 ------------------------------
 <p>
 <a class="source" href="/log/?revs=1-2">[1:2]</a>, <a class="source" href="/log/?revs=1-2">r1:2</a>, <a class="source" href="/log/?revs=12-23">[12:23]</a>, <a class="source" href="/log/?revs=12-23">r12:23</a>
 <a class="source" href="/log/trunk?revs=1-2">[1:2/trunk]</a>
+<a class="source" href="/log/trunk?revs=1-2">[2:1/trunk]</a> reversed
 </p>
 ------------------------------
 ============================== Escaping Log range TracLinks
@@ -156,9 +161,9 @@ log:trunk:12-head
 log:trunk:12@23 (bad, but shouldn't error out)
 ------------------------------
 <p>
-<a class="source" href="/log/?revs=12">log:@12</a>
+<a class="source" href="/log/?rev=12">log:@12</a>
 <a class="source" href="/log/trunk">log:trunk</a>
-<a class="source" href="/log/trunk?revs=12">log:trunk@12</a>
+<a class="source" href="/log/trunk?rev=12">log:trunk@12</a>
 <a class="source" href="/log/trunk?revs=12-23">log:trunk@12:23</a>
 <a class="source" href="/log/trunk?revs=12-23">log:trunk@12-23</a>
 <a class="source" href="/log/trunk?revs=12-23">log:trunk:12:23</a>
@@ -177,9 +182,9 @@ log:trunk@12?limit=10
 ------------------------------
 <p>
 <a class="source" href="/log/?limit=10">log:?limit=10</a>
-<a class="source" href="/log/?revs=12&amp;limit=10">log:@12?limit=10</a>
+<a class="source" href="/log/?rev=12&amp;limit=10">log:@12?limit=10</a>
 <a class="source" href="/log/trunk?limit=10">log:trunk?limit=10</a>
-<a class="source" href="/log/trunk?revs=12&amp;limit=10">log:trunk@12?limit=10</a>
+<a class="source" href="/log/trunk?rev=12&amp;limit=10">log:trunk@12?limit=10</a>
 <a class="source" href="/log/?revs=10-20&amp;verbose=yes&amp;format=changelog">[10:20?verbose=yes&amp;format=changelog]</a>
 <a class="source" href="/log/trunk?revs=10-20&amp;verbose=yes&amp;format=changelog">[10:20/trunk?verbose=yes&amp;format=changelog]</a>
 </p>
@@ -216,7 +221,7 @@ rfc:4180 should not be a log link
 """
 
 
-DIFF_TEST_CASES="""
+DIFF_TEST_CASES = """
 ============================== diff: link resolver
 diff:trunk//branch
 diff:trunk@12//branch@23
@@ -226,8 +231,8 @@ diff:@12:23
 <p>
 <a class="changeset" href="/changeset?new_path=branch&amp;old_path=trunk" title="Diff from trunk@latest to branch@latest">diff:trunk//branch</a>
 <a class="changeset" href="/changeset?new=23&amp;new_path=branch&amp;old=12&amp;old_path=trunk" title="Diff from trunk@12 to branch@23">diff:trunk@12//branch@23</a>
-<a class="changeset" href="/changeset?new=23&amp;new_path=trunk&amp;old=12&amp;old_path=trunk" title="Diff r12:23 for trunk">diff:trunk@12:23</a>
-<a class="changeset" href="/changeset?new=23&amp;old=12" title="Diff r12:23 for /">diff:@12:23</a>
+<a class="changeset" href="/changeset?new=23&amp;new_path=trunk&amp;old=12&amp;old_path=trunk" title="Diff [12:23] for trunk">diff:trunk@12:23</a>
+<a class="changeset" href="/changeset?new=23&amp;old=12" title="Diff [12:23] for /">diff:@12:23</a>
 </p>
 ------------------------------
 ============================== diff: link resolver + query
@@ -241,13 +246,13 @@ diff:trunk//branch?format=diff
 diff://
 ------------------------------
 <p>
-<a class="changeset" title="Diff rlatest:latest for /">diff://</a>
+<a class="changeset" title="Diff [latest:latest] for /">diff://</a>
 </p>
 ------------------------------
 """
 
 
-SOURCE_TEST_CASES="""
+SOURCE_TEST_CASES = """
 ============================== source: link resolver
 source:/foo/bar
 source:/foo/bar#42   # no long works as rev spec
@@ -301,16 +306,16 @@ export:123:/foo/pict.gif
 export:/foo/pict.gif@123
 ------------------------------
 <p>
-<a class="source" href="/export/200/foo/bar.html">export:/foo/bar.html</a>
-<a class="source" href="/export/123/foo/pict.gif">export:123:/foo/pict.gif</a>
-<a class="source" href="/export/123/foo/pict.gif">export:/foo/pict.gif@123</a>
+<a class="export" href="/export/HEAD/foo/bar.html">export:/foo/bar.html</a>
+<a class="export" href="/export/123/foo/pict.gif">export:123:/foo/pict.gif</a>
+<a class="export" href="/export/123/foo/pict.gif">export:/foo/pict.gif@123</a>
 </p>
 ------------------------------
 ============================== export: link resolver + fragment
 export:/foo/bar.html#header
 ------------------------------
 <p>
-<a class="source" href="/export/200/foo/bar.html#header">export:/foo/bar.html#header</a>
+<a class="export" href="/export/HEAD/foo/bar.html#header">export:/foo/bar.html#header</a>
 </p>
 ------------------------------
 """ # " (be Emacs friendly...)
