@@ -1,8 +1,7 @@
-import os
-import inspect
-import StringIO
-import unittest
 import difflib
+import os
+import re
+import unittest
 from datetime import datetime
 
 from trac.core import *
@@ -68,6 +67,15 @@ class NoneMacro(WikiMacroBase):
     def expand_macro(self, formatter, name, content):
         return None
 
+class WikiProcessorSampleMacro(WikiMacroBase):
+    def expand_macro(self, formatter, name, content, args):
+        if args is None:
+            return 'Called as a macro: ' + content
+        else:
+            return 'Called as a processor with params: <dl>%s</dl>' % \
+                ''.join('<dt>%s</dt><dd>%s</dd>' % kv for kv in args.items()) \
+                + content
+
 class SampleResolver(Component):
     """A dummy macro returning a div block, used by the unit test."""
 
@@ -92,23 +100,19 @@ class SampleResolver(Component):
 
 class WikiTestCase(unittest.TestCase):
 
-    def __init__(self, input, correct, file, line, setup=None, teardown=None,
-                 context=None):
+    generate_opts = {}
+
+    def __init__(self, title, input, correct, file, line, setup=None,
+                 teardown=None, context=None):
         unittest.TestCase.__init__(self, 'test')
-        self.title, self.input = input.split('\n', 1)
-        if self.title:
-            self.title = self.title.strip()
+        self.title = title
+        self.input = input
         self.correct = correct
         self.file = file
         self.line = line
         self._setup = setup
         self._teardown = teardown
 
-        self.env = EnvironmentStub()
-        wiki = WikiPage(self.env)
-        wiki.name = 'WikiStart'
-        wiki.text = '--'
-        wiki.save('joe', 'Entry page', '::1', datetime.now(utc))
         req = Mock(href=Href('/'), abs_href=Href('http://www.example.com/'),
                    authname='anonymous', perm=MockPerm(), args={})
         if context:
@@ -118,6 +122,11 @@ class WikiTestCase(unittest.TestCase):
             context = Context.from_request(req, 'wiki', 'WikiStart')
         self.context = context
 
+        all_test_components = [
+                HelloWorldMacro, DivHelloWorldMacro, TableHelloWorldMacro, 
+                DivCodeMacro, DivCodeElementMacro, DivCodeStreamMacro, 
+                NoneMacro, WikiProcessorSampleMacro, SampleResolver]
+        self.env = EnvironmentStub(enable=['trac.*'] + all_test_components)
         # -- macros support
         self.env.path = ''
         # -- intertrac support
@@ -137,22 +146,26 @@ class WikiTestCase(unittest.TestCase):
         self.env.abs_href = req.abs_href
 
     def setUp(self):
+        wiki = WikiPage(self.env)
+        wiki.name = 'WikiStart'
+        wiki.text = '--'
+        wiki.save('joe', 'Entry page', '::1', datetime.now(utc))
         if self._setup:
             self._setup(self)
 
     def tearDown(self):
+        self.env.reset_db()
         if self._teardown:
             self._teardown(self)
 
     def test(self):
         """Testing WikiFormatter"""
         formatter = self.formatter()
-        v = unicode(formatter.generate()).replace('\r','')
+        v = unicode(formatter.generate(**self.generate_opts)).replace('\r','')
         try:
             self.assertEquals(self.correct, v)
         except AssertionError, e:
             msg = to_unicode(e)
-            import re
             match = re.match(r"u?'(.*)' != u?'(.*)'", msg)
             if match:
                 g1 = ["%s\n" % x for x in match.group(1).split(r'\n')]
@@ -187,31 +200,57 @@ class OneLinerTestCase(WikiTestCase):
     def formatter(self):
         return InlineHtmlFormatter(self.env, self.context, self.input)
 
+class EscapeNewLinesTestCase(WikiTestCase):
+    generate_opts = {'escape_newlines': True}
+    def formatter(self):
+        return HtmlFormatter(self.env, self.context, self.input)
+
 
 def suite(data=None, setup=None, file=__file__, teardown=None, context=None):
     suite = unittest.TestSuite()
-    if not data:
-        file = os.path.join(os.path.split(file)[0], 'wiki-tests.txt')
-        data = open(file, 'r').read().decode('utf-8')
-    tests = data.split('=' * 30)
-    next_line = 1
-    line = 0
-    for test in tests:
-        if line != next_line:
-            line = next_line
-        if not test or test == '\n':
-            continue
-        next_line += len(test.split('\n')) - 1
-        blocks = test.split('-' * 30 + '\n')
-        if len(blocks) != 3:
-            continue
-        input, page, oneliner = blocks
-        tc = WikiTestCase(input, page, file, line, setup, teardown, context)
-        suite.addTest(tc)
-        if oneliner:
-            tc = OneLinerTestCase(input, oneliner[:-1], file, line,
-                                  setup, teardown, context)
-            suite.addTest(tc)
+    def add_test_cases(data, filename):
+        tests = re.compile('^(%s.*)$' % ('=' * 30), re.MULTILINE).split(data)
+        next_line = 1
+        line = 0
+        for title, test in zip(tests[1::2], tests[2::2]):
+            title = title.lstrip('=').strip()            
+            if line != next_line:
+                line = next_line
+            if not test or test == '\n':
+                continue
+            next_line += len(test.split('\n')) - 1
+            if 'SKIP' in title or 'WONTFIX' in title:
+                continue
+            blocks = test.split('-' * 30 + '\n')
+            page_escape_nl = oneliner = None
+            if len(blocks) < 4:
+                blocks.extend([None,] * (4 - len(blocks)))
+            input, page, oneliner, page_escape_nl = blocks[:4]
+            if page:
+                page = WikiTestCase(
+                    title, input, page, filename, line, setup,
+                    teardown, context)
+            if oneliner:
+                oneliner = OneLinerTestCase(
+                    title, input, oneliner[:-1], filename, line, setup,
+                    teardown, context)
+            if page_escape_nl:
+                page_escape_nl = EscapeNewLinesTestCase(
+                    title, input, page_escape_nl, filename, line, setup,
+                    teardown, context)
+            for tc in [page, oneliner, page_escape_nl]:
+                if tc:
+                    suite.addTest(tc)
+    if data:
+        add_test_cases(data, file)
+    else:
+        for f in ('wiki-tests.txt', 'wikicreole-tests.txt'):
+            testfile = os.path.join(os.path.split(file)[0], f)
+            if os.path.exists(testfile):
+                data = open(testfile, 'r').read().decode('utf-8')
+                add_test_cases(data, testfile)
+            else:
+                print 'no ', testfile
     return suite
 
 if __name__ == '__main__':
